<?php
/**
 * This file implements misc functions to be called from the templates.
 *
 * This file is part of the evoCore framework - {@link http://evocore.net/}
 * See also {@link http://sourceforge.net/projects/evocms/}.
 *
 * @copyright (c)2003-2006 by Francois PLANQUE - {@link http://fplanque.net/}
 * Parts of this file are copyright (c)2004-2006 by Daniel HAHLER - {@link http://thequod.de/contact}.
 *
 * {@internal License choice
 * - If you have received this file as part of a package, please find the license.txt file in
 *   the same folder or the closest folder above for complete license terms.
 * - If you have received this file individually (e-g: from http://cvs.sourceforge.net/viewcvs.py/evocms/)
 *   then you must choose one of the following licenses before using the file:
 *   - GNU General Public License 2 (GPL) - http://www.opensource.org/licenses/gpl-license.php
 *   - Mozilla Public License 1.1 (MPL) - http://www.opensource.org/licenses/mozilla1.1.php
 * }}
 *
 * {@internal Open Source relicensing agreement:
 * Daniel HAHLER grants Francois PLANQUE the right to license
 * Daniel HAHLER's contributions to this file and the b2evolution project
 * under any OSI approved OSS license (http://www.opensource.org/licenses/).
 * }}
 *
 * @package evocore
 *
 * {@internal Below is a list of authors who have contributed to design/coding of this file: }}
 * @author blueyed: Daniel HAHLER.
 * @author cafelog (team)
 * @author fplanque: Francois PLANQUE.
 *
 * @version $Id: _template.funcs.php,v 1.6.2.3 2006/06/16 21:20:09 fplanque Exp $
 */
if( !defined('EVO_MAIN_INIT') ) die( 'Please, do not access this page directly.' );


/**
 * Display a global title matching filter params
 *
 * Outputs the title of the category when you load the page with <code>?cat=</code>
 * Display "Archive Directory" title if it has been requested
 * Display "Last comments" title if these have been requested
 * Display "Statistics" title if these have been requested
 * Display "User profile" title if it has been requested
 *
 * @todo single month: Respect locales datefmt
 * @todo single post: posts do no get proper checking (wether they are in the requested blog or wether their permissions match user rights,
 * thus the title sometimes gets displayed even when it should not. We need to pre-query the ItemList instead!!
 * @todo make it complete with all possible params!
 *
 * @param string prefix to display if a title is generated
 * @param string suffix to display if a title is generated
 * @param string glue to use if multiple title elements are generated
 * @param string format to output, default 'htmlbody'
 * @param boolean display title? (if false: return)
 * @param boolean show the year as link to year's archive (in monthly mode)
 */
function request_title( $prefix = ' ', $suffix = '', $glue = ' - ', $format = 'htmlbody',
												$display = true, $linktoyeararchive = true, $blogurl = '', $params = '' )
{
	global $cat_modifier, $cat_array, $m, $w, $month, $p, $title, $preview, $ItemCache, $disp, $s, $author;

	$r = array();

	switch( $disp )
	{
		case 'arcdir':
			// We are requesting the archive directory:
			$r[] = T_('Archive Directory');
			break;

		case 'comments':
			// We are requesting the last comments:
			$r[] = T_('Last comments');
			break;

		case 'profile':
			// We are requesting the user profile:
			$r[] = T_('User profile');
			break;

		case 'subs':
			// We are requesting the subscriptions screen:
			$r[] = T_('Subscriptions');
			break;

		case 'msgform':
			// We are requesting the message form:
			$r[] = T_('Send an email message');
			break;

		default:
			// We are displaying (a) message(s)...
			if( $preview )
			{	// We are requesting a post preview:
				$r[] = T_('PREVIEW');
			}
			elseif( intval($p) )
			{	// We are requesting a specific post by ID:
				if( $Item = & $ItemCache->get_by_ID( $p, false ) )
				{
					$r[] = T_('Post details').': '.$Item->get('title');
				}
			}
			elseif( !empty( $title ) )
			{	// We are requesting a specific post by title:
				if( $Item = & $ItemCache->get_by_urltitle( $title, false ) )
				{
					$r[] = T_('Post details').': '.$Item->get('title');
				}
			}
			else
			{	// Multiple messages...

				// CATEGORIES:
				if( !empty($cat_array) )
				{ // We have requested specific categories...
					$cat_names = array();
					foreach( $cat_array as $cat_ID )
					{
						if( ($my_cat = get_the_category_by_ID( $cat_ID, false ) ) !== false )
						{ // It is almost never meaningful to die over an invalid cat when generating title
							$cat_names[] = $my_cat['cat_name'];
						}
					}
					if( $cat_modifier == '*' )
					{
						$cat_names_string = implode( ' + ', $cat_names );
					}
					else
					{
						$cat_names_string = implode( ', ', $cat_names );
					}
					if( !empty( $cat_names_string ) )
					{
						if( $cat_modifier == '-' )
						{
							$cat_names_string = T_('All but ').$cat_names_string;
							$r[] = T_('Categories').': '.$cat_names_string;
						}
						else
						{
							if( count($cat_array) > 1 )
								$r[] = T_('Categories').': '.$cat_names_string;
							else
								$r[] = T_('Category').': '.$cat_names_string;
						}
					}
				}


				// TIMEFRAME:
				if( !empty($m) )
				{	// We have asked for a specific timeframe:

					$my_year = substr($m,0,4);
					if( strlen($m) > 4 && isset($month[substr($m,4,2)]) )
					{ // We have requested a month too:
						$my_month = T_($month[substr($m,4,2)]);
					}
					else
					{
						$my_month = '';
					}
					// Requested a day?
					$my_day = substr($m,6,2);

					if( $format == 'htmlbody' && !empty( $my_month ) && $linktoyeararchive )
					{ // display year as link to year's archive
						$my_year = '<a href="' . archive_link( $my_year, '', '', '', false, $blogurl, $params ) . '">' . $my_year . '</a>';
					}

					$arch = T_('Archives for').': '.$my_month.' '.$my_year;

					if( !empty( $my_day ) )
					{	// We also want to display a day
						$arch .= ", $my_day";
					}

					if( !empty($w) && ($w>=0) ) // Note: week # can be 0
					{	// We also want to display a week number
						$arch .= ", week $w";
					}

					$r[] = $arch;
				}


				// KEYWORDS:
				if( !empty($s) )
				{
					$r[] = T_('Keyword(s)').': '.$s;
				}


				// AUTHORS:
				if( !empty($author) )
				{
					$r[] = T_('Author(s)').': '.$author;
				}

			}
	}


	/* TODO: once the MainList is moved to ItemList2, remove a lot from above and get them like this:
	// EXPERIMENTAL:
	global $MainList;
	$additional_titles = $MainList->get_title();
	$r = array_merge( $r, $additional_titles );
	*/

	if( !empty( $r ) )
	{ // We have something to display:
		$r = implode( $glue, $r );
		$r = $prefix.format_to_output( $r, $format ).$suffix;
		if( $display )
			echo $r;
		else
			return $r;
	}
}


/*
 * single_month_title(-)
 * arcdir_title(-)
 *
 * @movedTo _obsolete092.php
 */


/**
 * Create a link to archive, using either params or extra path info.
 *
 * @param string year
 * @param string month
 * @param string day
 * @param string week
 * @param boolean show or return
 * @param string link, instead of blogurl
 * @param string GET params for 'file'
 */
function archive_link( $year, $month, $day = '', $week = '', $show = true, $file = '', $params = '' )
{
	global $Settings;

	if( empty($file) )
		$link = get_bloginfo('blogurl');
	else
		$link = $file;

	if( (! $Settings->get('links_extrapath')) || (!empty($params)) )
	{	// We reference by Query: Dirty but explicit permalinks
		$link = url_add_param( $link, $params );
		$link = url_add_param( $link, 'm=' );
		$separator = '';
	}
	else
	{
		$link = trailing_slash( $link ); // there can already be a slash from a siteurl like 'http://example.com/'
		$separator = '/';
	}

	$link .= $year;

	if( !empty( $month ) )
	{
		$link .= $separator.zeroise($month,2);
		if( !empty( $day ) )
		{
			$link .= $separator.zeroise($day,2);
		}
	}
	elseif( $week !== '' )  // Note: week # can be 0 !
	{
		if( ! $Settings->get('links_extrapath') )
		{	// We reference by Query: Dirty but explicit permalinks
			$link = url_add_param( $link, 'w='.$week );
		}
		else
		{
			$link .= '/w'.zeroise($week,2);
		}
	}

	$link .= $separator;

	if( $show )
	{
		echo $link;
	}
	return $link;
}

/*
 nolog */
?>
